@@ -1,5 +1,5 @@
-# runtime.py
-# Copyright (C) 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
+# mako/runtime.py
+# Copyright (C) 2006-2012 the Mako authors and contributors <see AUTHORS file>
 #
 # This module is part of Mako and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -13,74 +13,72 @@ import __builtin__, inspect, sys
 class Context(object):
     """Provides runtime namespace, output buffer, and various
     callstacks for templates.
-    
-     See :ref:`runtime_toplevel` for detail on the usage of
+ 
+    See :ref:`runtime_toplevel` for detail on the usage of
     :class:`.Context`.
-    
+ 
      """
-    
+ 
     def __init__(self, buffer, **data):
         self._buffer_stack = [buffer]
-        
-        # original data, minus the builtins
-        self._orig = data
-        
-        # the context data which includes builtins
-        self._data = __builtin__.__dict__.copy()
-        self._data.update(data)
+ 
+        self._data = data
         self._kwargs = data.copy()
         self._with_template = None
         self._outputting_as_unicode = None
         self.namespaces = {}
-        
+ 
         # "capture" function which proxies to the 
         # generic "capture" function
         self._data['capture'] = util.partial(capture, self)
-        
+ 
         # "caller" stack used by def calls with content
         self.caller_stack = self._data['caller'] = CallerStack()
-        
+ 
     @property
     def lookup(self):
         """Return the :class:`.TemplateLookup` associated 
         with this :class:`.Context`.
-        
+ 
         """
         return self._with_template.lookup
-        
+ 
     @property
     def kwargs(self):
         """Return the dictionary of keyword argments associated with this
         :class:`.Context`.
-        
+ 
         """
         return self._kwargs.copy()
-    
+ 
     def push_caller(self, caller):
         """Pushes a 'caller' callable onto the callstack for
         this :class:`.Context`."""
-        
-        
+ 
+ 
         self.caller_stack.append(caller)
-        
+ 
     def pop_caller(self):
         """Pops a 'caller' callable onto the callstack for this
         :class:`.Context`."""
 
         del self.caller_stack[-1]
-        
+ 
     def keys(self):
         """Return a list of all names established in this :class:`.Context`."""
 
         return self._data.keys()
-        
+ 
     def __getitem__(self, key):
-        return self._data[key]
+        if key in self._data:
+            return self._data[key]
+        else:
+            return __builtin__.__dict__[key]
 
     def _push_writer(self):
         """push a capturing buffer onto this Context and return
         the new writer function."""
-        
+ 
         buf = util.FastEncodingBuffer()
         self._buffer_stack.append(buf)
         return buf.write
@@ -88,33 +86,35 @@ class Context(object):
     def _pop_buffer_and_writer(self):
         """pop the most recent capturing buffer from this Context 
         and return the current writer after the pop.
-        
+ 
         """
 
         buf = self._buffer_stack.pop()
         return buf, self._buffer_stack[-1].write
-        
+ 
     def _push_buffer(self):
         """push a capturing buffer onto this Context."""
-        
+ 
         self._push_writer()
-        
+ 
     def _pop_buffer(self):
         """pop the most recent capturing buffer from this Context."""
-        
+ 
         return self._buffer_stack.pop()
-        
+ 
     def get(self, key, default=None):
         """Return a value from this :class:`.Context`."""
-        
-        return self._data.get(key, default)
-        
+ 
+        return self._data.get(key, 
+                __builtin__.__dict__.get(key, default)
+                )
+ 
     def write(self, string):
         """Write a string to this :class:`.Context` object's
         underlying output buffer."""
-        
+ 
         self._buffer_stack[-1].write(string)
-        
+ 
     def writer(self):
         """Return the current writer function"""
 
@@ -124,24 +124,23 @@ class Context(object):
         c = Context.__new__(Context)
         c._buffer_stack = self._buffer_stack
         c._data = self._data.copy()
-        c._orig = self._orig
         c._kwargs = self._kwargs
         c._with_template = self._with_template
         c._outputting_as_unicode = self._outputting_as_unicode
         c.namespaces = self.namespaces
         c.caller_stack = self.caller_stack
         return c
-        
+ 
     def locals_(self, d):
         """create a new :class:`.Context` with a copy of this 
         :class:`Context`'s current state, updated with the given dictionary."""
-        
+ 
         if len(d) == 0:
             return self
         c = self._copy()
         c._data.update(d)
         return c
-        
+ 
     def _clean_inheritance_tokens(self):
         """create a new copy of this :class:`.Context`. with
         tokens related to inheritance state removed."""
@@ -167,15 +166,15 @@ class CallerStack(list):
         self.nextcaller = None
     def _pop_frame(self):
         self.nextcaller = self.pop()
-        
-        
+ 
+ 
 class Undefined(object):
     """Represents an undefined value in a template.
-    
+ 
     All template modules have a constant value 
     ``UNDEFINED`` present which is an instance of this
     object.
-    
+ 
     """
     def __str__(self):
         raise NameError("Undefined")
@@ -194,51 +193,42 @@ class _NSAttr(object):
                 return getattr(ns.module, key)
             else:
                 ns = ns.inherits
-        raise AttributeError(key)    
-    
+        raise AttributeError(key) 
+ 
 class Namespace(object):
     """Provides access to collections of rendering methods, which 
       can be local, from other templates, or from imported modules.
-      
+ 
       To access a particular rendering method referenced by a 
       :class:`.Namespace`, use plain attribute access::
-      
+ 
         ${some_namespace.foo(x, y, z)}
-        
+ 
       :class:`.Namespace` also contains several built-in attributes 
       described here.
-      
+ 
       """
-    
-    def __init__(self, name, context, module=None, 
-                            template=None, templateuri=None, 
+ 
+    def __init__(self, name, context, 
                             callables=None, inherits=None, 
                             populate_self=True, calling_uri=None):
         self.name = name
-        if module is not None:
-            mod = __import__(module)
-            for token in module.split('.')[1:]:
-                mod = getattr(mod, token)
-            self._module = mod
-        else:
-            self._module = None
-        if templateuri is not None:
-            self.template = _lookup_template(context, templateuri, calling_uri)
-            self._templateuri = self.template.module._template_uri
-        else:
-            self.template = template
-            if self.template is not None:
-                self._templateuri = self.template.module._template_uri
         self.context = context
         self.inherits = inherits
         if callables is not None:
             self.callables = dict([(c.func_name, c) for c in callables])
-        else:
-            self.callables = None
-        if populate_self and self.template is not None:
-            lclcallable, lclcontext = \
-                        _populate_self_namespace(context, self.template, self_ns=self)
-    
+
+    callables = ()
+
+    module = None
+    """The Python module referenced by this Namespace.
+
+    If the namespace references a :class:`.Template`, then
+    this module is the equivalent of ``template.module``,
+    i.e. the generated module for the template.
+
+    """
+
     template = None
     """The :class:`.Template` object referenced by this
         :class:`.Namespace`, if any.
@@ -247,7 +237,7 @@ class Namespace(object):
 
     context = None
     """The :class:`.Context` object for this namespace.
-    
+ 
     Namespaces are often created with copies of contexts that
     contain slightly different data, particularly in inheritance
     scenarios. Using the :class:`.Context` off of a :class:`.Namespace` one
@@ -255,63 +245,44 @@ class Namespace(object):
     one-another.
 
     """
-    
-    
-    @property
-    def module(self):
-        """The Python module referenced by this Namespace.
-        
-        If the namespace references a :class:`.Template`, then
-        this module is the equivalent of ``template.module``,
-        i.e. the generated module for the template.
+ 
+    filename = None
+    """The path of the filesystem file used for this
+    Namespace's module or template.
 
-        """
-        return self._module or self.template.module
-    
-    @property
-    def filename(self):
-        """The path of the filesystem file used for this
-        Namespace's module or template.
-      
-        If this is a pure module-based
-        Namespace, this evaluates to ``module.__file__``. If a
-        template-based namespace, it evaluates to the original
-        template file location.
-        
-        """
-        if self._module:
-            return self._module.__file__
-        else:
-            return self.template.filename
-    
-    @property
-    def uri(self):
-        """The uri for this Namespace's template.
-        
-        I.e. whatever was sent to :meth:`.TemplateLookup.get_template()`.
-        
-        This is the equivalent of :attr:`Template.uri`.
+    If this is a pure module-based
+    Namespace, this evaluates to ``module.__file__``. If a
+    template-based namespace, it evaluates to the original
+    template file location.
 
-        """
-        return self.template.uri
+    """
+ 
+    uri = None
+    """The uri for this Namespace's template.
 
-    @property
+    I.e. whatever was sent to :meth:`.TemplateLookup.get_template()`.
+
+    This is the equivalent of :attr:`Template.uri`.
+
+    """
+
+    _templateuri = None
+
+    @util.memoized_property
     def attr(self):
         """Access module level attributes by name. 
-        
+ 
         This accessor allows templates to supply "scalar"
         attributes which are particularly handy in inheritance
         relationships. See the example in
         :ref:`inheritance_toplevel`.
 
         """
-        if not hasattr(self, '_attr'):
-            self._attr = _NSAttr(self)
-        return self._attr
+        return _NSAttr(self)
 
     def get_namespace(self, uri):
         """Return a :class:`.Namespace` corresponding to the given uri.
-        
+ 
         If the given uri is a relative uri (i.e. it does not
         contain ia leading slash ``/``), the uri is adjusted to
         be relative to the uri of the namespace itself. This
@@ -327,68 +298,55 @@ class Namespace(object):
         expressions that were generated within the body code of
         the template, or to conditionally use a particular
         namespace.
-        
+ 
         """
         key = (self, uri)
-        if self.context.namespaces.has_key(key):
+        if key in self.context.namespaces:
             return self.context.namespaces[key]
         else:
-            ns = Namespace(uri, self.context._copy(), 
+            ns = TemplateNamespace(uri, self.context._copy(), 
                                 templateuri=uri, 
                                 calling_uri=self._templateuri) 
             self.context.namespaces[key] = ns
             return ns
-    
+ 
     def get_template(self, uri):
         """Return a :class:`.Template` from the given uri.
-        
+ 
         The uri resolution is relative to the uri of this :class:`.Namespace`
         object's :class:`.Template`.
-        
+ 
         """
         return _lookup_template(self.context, uri, self._templateuri)
-        
+ 
     def get_cached(self, key, **kwargs):
         """Return a value from the :class:`.Cache` referenced by this 
         :class:`.Namespace` object's :class:`.Template`.
-        
+ 
         The advantage to this method versus direct access to the 
         :class:`.Cache` is that the configuration parameters
         declared in ``<%page>`` take effect here, thereby calling
         up the same configured backend as that configured
         by ``<%page>``.
-        
+ 
         """
-        
-        if self.template:
-            if not self.template.cache_enabled:
-                createfunc = kwargs.get('createfunc', None)
-                if createfunc:
-                    return createfunc()
-                else:
-                    return None
-                
-            if self.template.cache_dir:
-                kwargs.setdefault('data_dir', self.template.cache_dir)
-            if self.template.cache_type:
-                kwargs.setdefault('type', self.template.cache_type)
-            if self.template.cache_url:
-                kwargs.setdefault('url', self.template.cache_url)
+
         return self.cache.get(key, **kwargs)
-    
+ 
     @property
     def cache(self):
-        """Return the :class:`.Cache` object referenced by this :class:`.Namespace` object's
+        """Return the :class:`.Cache` object referenced 
+        by this :class:`.Namespace` object's
         :class:`.Template`.
-        
+ 
         """
         return self.template.cache
-    
+ 
     def include_file(self, uri, **kwargs):
         """Include a file at the given uri"""
-        
+ 
         _include_file(self.context, uri, self._templateuri, **kwargs)
-        
+ 
     def _populate(self, d, l):
         for ident in l:
             if ident == '*':
@@ -396,51 +354,164 @@ class Namespace(object):
                     d[k] = v
             else:
                 d[ident] = getattr(self, ident)
-    
+ 
     def _get_star(self):
         if self.callables:
             for key in self.callables:
                 yield (key, self.callables[key])
-        if self.template:
-            def get(key):
-                callable_ = self.template._get_def_callable(key)
-                return util.partial(callable_, self.context)
-            for k in self.template.module._exports:
-                yield (k, get(k))
-        if self._module:
-            def get(key):
-                callable_ = getattr(self._module, key)
-                return util.partial(callable_, self.context)
-            for k in dir(self._module):
-                if k[0] != '_':
-                    yield (k, get(k))
-                            
+
     def __getattr__(self, key):
-        if self.callables and key in self.callables:
-            return self.callables[key]
-
-        if self.template and self.template.has_def(key):
-            callable_ = self.template._get_def_callable(key)
-            return util.partial(callable_, self.context)
-
-        if self._module and hasattr(self._module, key):
-            callable_ = getattr(self._module, key)
-            return util.partial(callable_, self.context)
-
-        if self.inherits is not None:
-            return getattr(self.inherits, key)
-        raise AttributeError(
+        if key in self.callables:
+            val = self.callables[key]
+        elif self.inherits:
+            val = getattr(self.inherits, key)
+        else:
+            raise AttributeError(
                     "Namespace '%s' has no member '%s'" % 
                     (self.name, key))
+        setattr(self, key, val)
+        return val
+
+class TemplateNamespace(Namespace):
+    """A :class:`.Namespace` specific to a :class:`.Template` instance."""
+
+    def __init__(self, name, context, template=None, templateuri=None, 
+                            callables=None, inherits=None, 
+                            populate_self=True, calling_uri=None):
+        self.name = name
+        self.context = context
+        self.inherits = inherits
+        if callables is not None:
+            self.callables = dict([(c.func_name, c) for c in callables])
+
+        if templateuri is not None:
+            self.template = _lookup_template(context, templateuri, 
+                                                calling_uri)
+            self._templateuri = self.template.module._template_uri
+        elif template is not None:
+            self.template = template
+            self._templateuri = template.module._template_uri
+        else:
+            raise TypeError("'template' argument is required.")
+
+        if populate_self:
+            lclcallable, lclcontext = \
+                        _populate_self_namespace(context, self.template, 
+                                                    self_ns=self)
+
+    @property
+    def module(self):
+        """The Python module referenced by this Namespace.
+ 
+        If the namespace references a :class:`.Template`, then
+        this module is the equivalent of ``template.module``,
+        i.e. the generated module for the template.
+
+        """
+        return self.template.module
+
+    @property
+    def filename(self):
+        """The path of the filesystem file used for this
+        Namespace's module or template.
+        """
+        return self.template.filename
+
+    @property
+    def uri(self):
+        """The uri for this Namespace's template.
+ 
+        I.e. whatever was sent to :meth:`.TemplateLookup.get_template()`.
+ 
+        This is the equivalent of :attr:`Template.uri`.
+
+        """
+        return self.template.uri
+
+    def _get_star(self):
+        if self.callables:
+            for key in self.callables:
+                yield (key, self.callables[key])
+        def get(key):
+            callable_ = self.template._get_def_callable(key)
+            return util.partial(callable_, self.context)
+        for k in self.template.module._exports:
+            yield (k, get(k))
+
+    def __getattr__(self, key):
+        if key in self.callables:
+            val = self.callables[key]
+        elif self.template.has_def(key):
+            callable_ = self.template._get_def_callable(key)
+            val = util.partial(callable_, self.context)
+        elif self.inherits:
+            val = getattr(self.inherits, key)
+
+        else:
+            raise AttributeError(
+                    "Namespace '%s' has no member '%s'" % 
+                    (self.name, key))
+        setattr(self, key, val)
+        return val
+
+class ModuleNamespace(Namespace):
+    """A :class:`.Namespace` specific to a Python module instance."""
+
+    def __init__(self, name, context, module, 
+                            callables=None, inherits=None, 
+                            populate_self=True, calling_uri=None):
+        self.name = name
+        self.context = context
+        self.inherits = inherits
+        if callables is not None:
+            self.callables = dict([(c.func_name, c) for c in callables])
+
+        mod = __import__(module)
+        for token in module.split('.')[1:]:
+            mod = getattr(mod, token)
+        self.module = mod
+
+    @property
+    def filename(self):
+        """The path of the filesystem file used for this
+        Namespace's module or template.
+        """
+        return self.module.__file__
+
+    def _get_star(self):
+        if self.callables:
+            for key in self.callables:
+                yield (key, self.callables[key])
+        def get(key):
+            callable_ = getattr(self.module, key)
+            return util.partial(callable_, self.context)
+        for k in dir(self.module):
+            if k[0] != '_':
+                yield (k, get(k))
+
+    def __getattr__(self, key):
+        if key in self.callables:
+            val = self.callables[key]
+        elif hasattr(self.module, key):
+            callable_ = getattr(self.module, key)
+            val = util.partial(callable_, self.context)
+        elif self.inherits:
+            val = getattr(self.inherits, key)
+        else:
+            raise AttributeError(
+                    "Namespace '%s' has no member '%s'" % 
+                    (self.name, key))
+        setattr(self, key, val)
+        return val
 
 def supports_caller(func):
     """Apply a caller_stack compatibility decorator to a plain
     Python function.
-    
+ 
     See the example in :ref:`namespaces_python_modules`.
-    
+ 
     """
-    
+ 
     def wrap_stackframe(context,  *args, **kwargs):
         context.caller_stack._push_frame()
         try:
@@ -448,15 +519,15 @@ def supports_caller(func):
         finally:
             context.caller_stack._pop_frame()
     return wrap_stackframe
-        
+ 
 def capture(context, callable_, *args, **kwargs):
     """Execute the given template def, capturing the output into
     a buffer.
-    
+ 
     See the example in :ref:`namespaces_python_modules`.
-    
+ 
     """
-    
+ 
     if not callable(callable_):
         raise exceptions.RuntimeException(
                             "capture() function expects a callable as "
@@ -482,7 +553,7 @@ def _decorate_toplevel(fn):
             return fn(y)(context, *args, **kw)
         return go
     return decorate_render
-    
+ 
 def _decorate_inline(context, fn):
     def decorate_render(render_fn):
         dec = fn(render_fn)
@@ -490,17 +561,17 @@ def _decorate_inline(context, fn):
             return dec(context, *args, **kw)
         return go
     return decorate_render
-            
+ 
 def _include_file(context, uri, calling_uri, **kwargs):
     """locate the template from the given uri and include it in
     the current output."""
-    
+ 
     template = _lookup_template(context, uri, calling_uri)
     (callable_, ctx) = _populate_self_namespace(
                                 context._clean_inheritance_tokens(), 
                                 template)
-    callable_(ctx, **_kwargs_for_include(callable_, context._orig, **kwargs))
-        
+    callable_(ctx, **_kwargs_for_include(callable_, context._data, **kwargs))
+ 
 def _inherit_from(context, uri, calling_uri):
     """called by the _inherit method in template modules to set
     up the inheritance chain at the start of a template's
@@ -514,7 +585,7 @@ def _inherit_from(context, uri, calling_uri):
     while ih.inherits is not None:
         ih = ih.inherits
     lclcontext = context.locals_({'next':ih})
-    ih.inherits = Namespace("self:%s" % template.uri, 
+    ih.inherits = TemplateNamespace("self:%s" % template.uri, 
                                 lclcontext, 
                                 template = template, 
                                 populate_self=False)
@@ -544,7 +615,7 @@ def _lookup_template(context, uri, relativeto):
 
 def _populate_self_namespace(context, template, self_ns=None):
     if self_ns is None:
-        self_ns = Namespace('self:%s' % template.uri, 
+        self_ns = TemplateNamespace('self:%s' % template.uri, 
                                 context, template=template, 
                                 populate_self=False)
     context._data['self'] = context._data['local'] = self_ns
@@ -560,27 +631,27 @@ def _render(template, callable_, args, data, as_unicode=False):
 
     if as_unicode:
         buf = util.FastEncodingBuffer(unicode=True)
-    elif template.output_encoding:
+    elif template.bytestring_passthrough:
+        buf = util.StringIO()
+    else:
         buf = util.FastEncodingBuffer(
                         unicode=as_unicode, 
                         encoding=template.output_encoding, 
                         errors=template.encoding_errors)
-    else:
-        buf = util.StringIO()
     context = Context(buf, **data)
     context._outputting_as_unicode = as_unicode
     context._with_template = template
-    
+ 
     _render_context(template, callable_, context, *args, 
                             **_kwargs_for_callable(callable_, data))
     return context._pop_buffer().getvalue()
 
 def _kwargs_for_callable(callable_, data):
-    argspec = inspect.getargspec(callable_)
+    argspec = util.inspect_func_args(callable_)
     # for normal pages, **pageargs is usually present
     if argspec[2]:
         return data
-    
+ 
     # for rendering defs from the top level, figure out the args
     namedargs = argspec[0] + [v for v in argspec[1:3] if v is not None]
     kwargs = {}
@@ -590,13 +661,13 @@ def _kwargs_for_callable(callable_, data):
     return kwargs
 
 def _kwargs_for_include(callable_, data, **kwargs):
-    argspec = inspect.getargspec(callable_)
+    argspec = util.inspect_func_args(callable_)
     namedargs = argspec[0] + [v for v in argspec[1:3] if v is not None]
     for arg in namedargs:
         if arg != 'context' and arg in data and arg not in kwargs:
             kwargs[arg] = data[arg]
     return kwargs
-    
+ 
 def _render_context(tmpl, callable_, context, *args, **kwargs):
     import mako.template as template
     # create polymorphic 'self' namespace for this 
@@ -609,7 +680,7 @@ def _render_context(tmpl, callable_, context, *args, **kwargs):
         # otherwise, call the actual rendering method specified
         (inherit, lclcontext) = _populate_self_namespace(context, tmpl.parent)
         _exec_template(callable_, context, args=args, kwargs=kwargs)
-        
+ 
 def _exec_template(callable_, context, args=None, kwargs=None):
     """execute a rendering callable given the callable, a
     Context, and optional explicit arguments
@@ -626,7 +697,7 @@ def _exec_template(callable_, context, args=None, kwargs=None):
             callable_(context, *args, **kwargs)
         except Exception, e:
             _render_error(template, context, e)
-        except:                
+        except: 
             e = sys.exc_info()[0]
             _render_error(template, context, e)
     else:
@@ -645,6 +716,6 @@ def _render_error(template, context, error):
             context._buffer_stack[:] = [util.FastEncodingBuffer(
                                             error_template.output_encoding,
                                             error_template.encoding_errors)]
-                                            
+ 
         context._with_template = error_template
         error_template.render_context(context, error=error)
